
/*
 * This file is part of the netcc project
 *
 * (C) 2011 Andreas Steinmetz, ast@domdv.de
 * The contents of this file is licensed under the GPL version 2.
 */

#include "headers.h"
#include "md5.h"
#include "common.h"
#include "clntlog.h"

/*
 * clienterr
 *
 * input: see printf()
 *
 * description:	This routine logs the given message either to stderr or
 *		a log file. To support concurrency the log file, if any
 *		is locked during append.
 */

void clienterr(char *fmt,...)
{
	struct flock lock={F_WRLCK,SEEK_SET,0,0,0};
	char *logfile=getenv(LOGNAME);
	FILE *logfd;
	va_list ap;

	/* either use stderr or open and lock the log file */

	va_start(ap,fmt);
	if(LIKELY(!logfile))logfd=stderr;
	else if(UNLIKELY((logfd=fopen(logfile,"a"))==NULL))
	{
		fprintf(logfd,"%s %s %d: can't write to %s\n",app,
			__FILE__,__LINE__,logfile);
		logfile=NULL;
		logfd=stderr;
	}
	else
	{
		fcntl(fileno(logfd),F_SETLKW,&lock);
		fseek(logfd,0,SEEK_END);
	}

	/* output the message */

	vfprintf(logfd,fmt,ap);
	fflush(logfd);

	/* unlock and close the log file, if any */

	if(logfile)
	{
		lock.l_type=F_UNLCK;
		fcntl(fileno(logfd),F_SETLK,&lock);
		fclose(logfd);
	}
	va_end(ap);
}
