
/*
 * This file is part of the netcc project
 *
 * (C) 2011 Andreas Steinmetz, ast@domdv.de
 * The contents of this file is licensed under the GPL version 2.
 */

#ifndef COMMON_H
#define COMMON_H

/* cpp dependent definitions */

#define STR(s)		XSTR(s)
#define XSTR(s)		#s

/* compiler dependent definitions */

#define NORETURN	__attribute__((noreturn))
#define LIKELY(a)	__builtin_expect((a),1)
#define UNLIKELY(a)	__builtin_expect((a),0)
#define UNUSED(a)	a __attribute__((unused))

/* compiler dependent tool names */

#define CPP		"cpp"
#define CC		"cc"
#define GCC		"gcc"
#define CXX		"c++"
#define GXX		"g++"
#define VOPT		"-dumpversion"

/* networking definition */

#define SWITCHES_SUPPORT_MLDV2(a) UNLIKELY(a)
#define MSITEADDR6	"ff15::dead:beef"
#define MCASTADDR6	"ff12::dead:beef"
#define MCASTADDR4	"232.97.115.116"
#define OOBINTERVAL	30
#define NETWAIT		5

/* environment variable names */

#define LOGNAME		"NETCC_LOG"
#define SOCKNAME	"NETCC_SOCKET"
#define MASQNAME	"NETCC_MASQDIR"
#define VERBOSE		"NETCC_VERBOSE"
#define DBGFLAG		"NETCC_DEBUG"
#define TIMEOUT		"NETCC_TIMEOUT"
#define LOCALFLAG	"NETCC_LOCAL"
#define ALLOWNAME	"NETCC_ALLOW"
#define STRIPNAME	"NETCC_STRIP"
#define WALOCALFLAG	"NETCC_WALOCAL"

/* platform dependet definitions */

#if defined(__linux__)
#define FIFO		"none"
#define DEVICE		"eth0"
#define TMPDIR		"/dev/shm"
#define ROOTGRP		"root"
#elif defined(__FreeBSD__) || defined(__OpenBSD__)
#define FIFO		"/var/run/netccd.fifo"
#define DEVICE		"em0"
#define TMPDIR		"/var/tmp"
#define ROOTGRP		"wheel"
#elif defined(__NetBSD__)
#define FIFO		"/var/run/netccd.fifo"
#define DEVICE		"wm0"
#define TMPDIR		"/var/tmp"
#define ROOTGRP		"wheel"
#else
#error os dependent definitions missing
#endif

/* default path */

#define DFTPATH		"/usr/local/bin:/usr/bin:/bin"

/* default allowed remote tools */

#define ALLOWEDRMT	"cc:c++:gcc:g++"

/* maximum number of allowed tools list */

#define MAXTOOLS	32

/* various defaults */

#define WINDOW		30
#define MINFREE		128
#define SLOWCPU		1800
#define MAXJOBS		128
#define RMTUSER		"nobody"
#define RMTGROUP	"nobody"
#define SOCKET		"/var/run/netccd.sock"
#define PIDFILE		"/var/run/netccd.pid"
#define SVPIDFILE	"/var/run/netccsvd.pid"
#define PORT		3634
#define SVPORT		3635
#define NICE		0
#define NETCCD		"netccd"
#define NETCCSVD	"netccsvd"
#define NETCCDCTL	"netccd-ctl"
#define NETCC		"netcc"
#define NOPRIVUSR	"nobody"
#define NOPRIVGRP	"nobody"

/* hint flags */

#define HINT_S		0x00000001
#define HINT_STDC	0x00000002
#define HINT_CXX	0x00000004
#define HINT_OBJC	0x00000008
#define HINT_OBJCXX	0x00000010
#define HINT_ADDSWITCH	0x00000020
#define HINT_USECORK	0x00000040
#define HINT_ZLIB	0x00000080
#define HINT_OOB	0x00000100
#define HINT_RMASK	0x00000fff
#define HINT_C		0x00001000
#define HINT_ASM	0x00002000
#define HINT_OUT	0x00004000
#define HINT_IN		0x00008000
#define HINT_CPP	0x00010000
#define HINT_ZAP	0x00020000
#define HINT_IPV6	0x00040000
#define HINT_SCTP	0x00080000
#define HINT_SPLICE	0x00100000
#define HINT_SCTPAUTH	0x00200000
#define HINT_SCTPBUG	0x00400000
#define HINT_CMASK	0x00fff000
#define HINT_LIGHT	0x01000000
#define HINT_AUTH	0x02000000
#define HINT_GROUP1	0x04000000
#define HINT_G0ACCESS	0x08000000
#define HINT_G1ACCESS	0x10000000
#define HINT_UMASK	0xff000000

/* ipc readability */

#define JOBCOUNT	COUNTER_0
#define JOBCOND		COND_0
#define CPPCOUNT	COUNTER_1
#define CPPCOND		COND_1

/* response message (netccd to netcc) */

typedef struct
{
	int addr;		/* IPv4 address or IPv6 interface index	*/
	int port;		/* target port				*/
	int ttl;		/* ttl to use				*/
	int nice;		/* nice value to use			*/
	int extra;		/* hints				*/
	int unique;		/* remote system unique id		*/
	struct in6_addr addr6;	/* IPv6 address				*/
} MSG;

/* request type ids */

#define LOCAL		0	/* local compile			*/
#define ANYWHERE	1	/* anywhere compile			*/
#define NOSERVICE	2	/* broken remote system notify		*/
#define CONTROL		3	/* control utility request		*/

/* request message (netcc and netccd-ctl to netccd */

typedef struct
{
	int where;		/* request type id			*/
	int extra;		/* hints or IPv4 address		*/
	struct in6_addr addr6;	/* IPv6 address				*/
} REQ;

/* control utility processing request flags */

#define MAP_FASTCPU	0x0001
#define MAP_CLIENTONLY	0x0002
#define MAP_LOCALONLY	0x0004
#define MAP_NICE	0x0008
#define MAP_DEVICE	0x0010
#define MAP_BCASTSPEED	0x0020
#define MAP_SYSLOAD	0x0040
#define MAP_ZLIB	0x0080
#define MAP_CORK	0x0100

/* control utility request data (read the source, Luke) */

typedef struct
{
	int map;
	int bogoslow;
	int clientonly;
	int justlocal;
	int nice;
	int slowbcast;
	int lightload;
	int zlib;
	int cork;
	char dev[256];
} CTLDATA;

/* reply to control utility request */

typedef struct
{
	int map;
} CTLSTATUS;

/* network processing mode */

#define	LAN_BCAST	0	/* IPv4 broadcast mode	*/
#define LAN_MCAST	1	/* link local multicast	*/
#define SITE_MCAST	2	/* sitewide multicast	*/

/* authenticated header format */

typedef struct
{
	char auth[MD5_SIZE+8];	/* auth header		*/
	uint32_t data[4];	/* state dependent data	*/
} THDR;

/* shared memory element sizes */

#define CLPATH		1024
#define CLRENV		1024
#define CLTMPD		1024

/* netccd to netcc shared memory format */

typedef struct
{
	int window;
	MD5HMDATA key;
	MD5(hash);
	char path[CLPATH];
	char clrenv[CLRENV];
	char tmpdir[CLTMPD];
} SHM;

/* the joys of *BSD, some are less equal than the others... */

#if defined(__NetBSD__)
#define EMPTY	0
#define PUT(a)	((long)(a))
#define GET(a)	((void *)(a))
#else
#define EMPTY	NULL
#define PUT(a)	(a)
#define GET(a)	(a)
#endif

#endif
