
/*
 * This file is part of the netcc project
 *
 * (C) 2011 Andreas Steinmetz, ast@domdv.de
 * The contents of this file is licensed under the GPL version 2.
 */

#include "headers.h"
#include "md5.h"
#include "common.h"
#include "fqdn.h"
#include "log.h"

static char *fqdn=NULL;

/*
 * setfqdn
 *
 * input:  f
 *
 * return: 0 in case of success, -1 in case of error
 *
 * description:	This routine sets the user specified fqdn.
 */

int setfqdn(char *f)
{
	if(!f||!*f||strlen(f)>63)BAD("illegal fqdn supplied");
	if(UNLIKELY(!(fqdn=strdup(f))))BAD("malloc failure");
	return 0;
}

/*
 * getfqdn
 *
 * return: pointer to an allocated fqdn string or NULL in case of error
 *
 * description:	This routine returns either the user specified fqdn or
 *		it retrieves and returns the systme fqdn.
 */

char *getfqdn(void)
{
	char *p;
	char h[256];
	char d[256];
	char f[512];

	if(fqdn)
	{
		if(UNLIKELY(!(p=strdup(fqdn))))NOTICE("malloc failure");
		return p;
	}

	if(UNLIKELY(gethostname(h,sizeof(h)-1)))
	{
		NOTICE("gethostname: %s",errstr(errno,f));
		return NULL;
	}
	h[sizeof(h)-1]=0;

	if(UNLIKELY(getdomainname(d,sizeof(d)-1)))
	{
		NOTICE("getdomainname: %s",errstr(errno,f));
		return NULL;
	}
	d[sizeof(d)-1]=0;

	strcpy(f,h);
	if(LIKELY(*d))if(strcmp(d,"(none)"))
	{
		strcat(f,".");
		strcat(f,d);
	}

	if(UNLIKELY(!(p=strdup(f))))NOTICE("malloc failure");

	return p;
}
