#!/bin/sh
#
# This file is part of the netcc project
#
# (C) 2011 Andreas Steinmetz, ast@domdv.de
# The contents of this file is licensed under the GPL version 2.
#
# -------------------------------------------------------------------
#
# linux script to create a private ipv6 network range which should be
# in conformance with rfc 4193.
#
# Usage:
# genprivate.sh <your-network-interface> <ntp-server-address>
#
export PATH=/sbin:/bin:/usr/sbin:/usr/bin
ifc="$1"
ntp="$2"
if [ "$ifc" = "" -o "$ntp" = "" ]
then
	echo "Usage: $0 <your-network-interface> <ntp-server-address>"
	exit 1
fi
macuid="`ifconfig $ifc 2> /dev/null | head -1 | sed -e 's/.*HWaddr //' -e 's/\(..:..:..\):\(..:..:..\)/\1:ff:fe:\2/' -e 's/:/ /g'`"
if [ "$macuid" = "" ]
then
	echo "Error processing MAC address of $ifc"
	exit 1
fi
subnetid="`dd if=/dev/urandom bs=1 count=2 2> /dev/null | od -tx2 | head -1 | cut -d' ' -f2`"
if [ "$subnetid" = "" ]
then
	echo "Error reading /dev/urandom for subnet id"
	exit 1
fi
ntptime="`ntpdate -d $ntp 2> /dev/null | grep '^originate timestamp' | cut -d' ' -f3 | sed 's/\(..\)\(..\)\(..\)\(..\)\.\(..\)\(..\)\(..\)\(..\)/\1 \2 \3 \4 \5 \6 \7 \8/'`"
if [ "$ntptime" = "" ]
then
	echo "Error getting NTP time of $ntp"
	exit 1
fi
globalid="`echo "0: $ntptime $macuid" | xxd -r | sha1sum -b - | sed -e 's/^.\{30\}//' -e 's/ .*//'`"
privatenet=`echo "fd$globalid$subnetid" | sed 's/^\(....\)\(....\)\(....\)\(....\)/\1:\2:\3:\4::\/64/'`
echo "Your private IPv6 network is: $privatenet"
