
/*
 * This file is part of the netcc project
 *
 * (C) 2011 Andreas Steinmetz, ast@domdv.de
 * The contents of this file is licensed under the GPL version 2.
 */

#ifndef LOG_H
#define LOG_H

/* various logging macros */

#define DOLOG(a,b,...)	syslog(a,"%s %d: " b,__FILE__,__LINE__,##__VA_ARGS__)
#define ERR(a,...)	DOLOG(LOG_ERR,a,##__VA_ARGS__)
#define WARN(a,...)	DOLOG(LOG_WARNING,a,##__VA_ARGS__)
#define NOTICE(a,...)	DOLOG(LOG_NOTICE,a,##__VA_ARGS__)
#define INFO(a,...)	DOLOG(LOG_INFO,a,##__VA_ARGS__)
#define DEBUG(a,...)	DOLOG(LOG_DEBUG,a,##__VA_ARGS__)

#define FAIL(a,...)	{ERR(a,##__VA_ARGS__);return 1;}
#define BAD(a,...)	{ERR(a,##__VA_ARGS__);return -1;}
#define FJMP(a,b,...)	{ERR(b,##__VA_ARGS__);goto a;}
#define GOTO(a,b,...)	{WARN(b,##__VA_ARGS__);goto a;}
#define JUMP(a,b,...)	{NOTICE(b,##__VA_ARGS__);goto a;}

/*
 * _errstr
 *
 * description:	System independent inline to retrieve the error message
 *		related to an errno value. Must only be called via the
 *		errstr() macro.
 */

static inline char *_errstr(int err,char *buf,int len)
{
#ifdef _GNU_SOURCE
	return strerror_r(err,buf,len);
#else
	*buf=0;
	strerror_r(err,buf,len);
	return buf;
#endif
}

#define errstr(err,buf)	_errstr(err,buf,sizeof(buf))

#endif

