
/*
 * This file is part of the netcc project
 *
 * (C) 2011 Andreas Steinmetz, ast@domdv.de
 * The contents of this file is licensed under the GPL version 2.
 */

#ifndef MD5_H
#define MD5_H

/* sanity settings */

#if !defined(MD5_FULL) && !defined(MD5_FAST) && !defined(MD5_HMAC_FULL) && !defined(MD5_HMAC_FAST)
#define MD5_FULL
#define MD5_FAST
#define MD5_HMAC_FULL
#define MD5_HMAC_FAST
#define MD5_PARANOID
#endif

#if defined(MD5_HMAC_FULL) || defined(MD5_HMAC_FAST)
#ifndef MD5_HMAC
#define MD5_HMAC
#endif
#ifndef MD5_PARANOID
#define MD5_PARANOID
#endif
#endif

/* definitions */

#define MD5_SIZE 16
#define MD5(a) uint8_t a[MD5_SIZE]

/* required types */

typedef struct
{
	uint32_t md5[4];/* current md5 result				*/
	uint32_t total;	/* total amount of bytes processed		*/
	union
	{
	  uint32_t l[16];/* longword buffer for paranoia memory reset	*/
	  uint8_t b[64];/* buffer for incomplete 64 byte message block	*/
	}bfr;
	uint8_t size;	/* size of data in buffer			*/
} MD5DATA;

#ifdef MD5_HMAC
typedef struct
{
	uint32_t imd5[4];/* preprocessed HMAC key (inner)		*/
	uint32_t omd5[4];/* preprocessed HMAC key (outer)		*/
} MD5HMDATA;
#endif

/* function prototypes */

#if defined(MD5_FULL) || defined(MD5_HMAC)
extern void md5init(MD5DATA *ptr);
extern void md5next(uint8_t *data,uint32_t length,MD5DATA *ptr);
extern void md5end(uint8_t *result,MD5DATA *ptr);
#endif
#ifdef MD5_FAST
extern void md5(uint8_t *data,uint32_t length,uint8_t *result);
#endif
#if defined(MD5_HMAC_FULL) || defined(MD5_HMAC_FAST)
extern void md5hmkey(uint8_t *key,uint32_t keylength,MD5HMDATA *ptr);
#endif
#ifdef MD5_HMAC_FULL
extern void md5hminit(MD5DATA *ptr,MD5HMDATA *key);
#define md5hmnext(a,b,c) md5next(a,b,c)
extern void md5hmend(uint8_t *result,MD5DATA *ptr,MD5HMDATA *key);
#endif
#ifdef MD5_HMAC_FAST
extern void md5hmac(uint8_t *data,uint32_t length,uint8_t *result,
	MD5HMDATA *key);
#endif

#endif
