
/*
 * This file is part of the netcc project
 *
 * (C) 2011 Andreas Steinmetz, ast@domdv.de
 * The contents of this file is licensed under the GPL version 2.
 */

#include "headers.h"
#include "md5.h"
#include "common.h"
#include "safeapi.h"

#ifdef SAFEAPI

int safeapi=0;

#else
#warning no fork safe fd related functions available, need lots of locks
#endif

/*
 * safeapi
 *
 * return: 0 in case of success, -1 in case of error
 *
 * description:	This routine determines, if the lockless mode can be
 *		used (requires atomic close on exec setting available
 *		for all operations returning a file descriptor).
 */

int enablesafeapi(void)
{
#if defined(SAFEAPI) && defined(__linux__)
	int d;
	char *save;
	char *num;
	struct utsname bfr;

	/* get system information */

	if(uname(&bfr))return -1;

	/* handle system major */

	if(UNLIKELY(!(num=strtok_r(bfr.release,".",&save))))return -1;
	if((d=atoi(num))>2)
	{
		safeapi=1;
		return 0;
	}
	else if(d<2)return 0;

	/* handle system minor */

	if(UNLIKELY(!(num=strtok_r(NULL,".",&save))))return -1;
	if((d=atoi(num))>6)
	{
		safeapi=1;
		return 0;
	}
	else if(d<6)return 0;

	/* handle system patch level */

	if(UNLIKELY(!(num=strtok_r(NULL,".",&save))))return -1;
	if((d=atoi(num))>=27)
	{
		safeapi=1;
		return 0;
	}
#endif
	/* signal success */

	return 0;
}

/* this is a warning for all the more ancient systems */

#ifndef SAFEUID
#warning no per thread uid/gid setting for fs access, need lots of locks
#endif
