
/*
 * This file is part of the netcc project
 *
 * (C) 2011 Andreas Steinmetz, ast@domdv.de
 * The contents of this file is licensed under the GPL version 2.
 */

#include "headers.h"
#include "md5.h"
#include "common.h"
#include "unix.h"
#include "clntlog.h"

/*
 * unixconnext
 *
 * input:  n - unix domain socket pathname
 *
 * return: a socket descriptor or -1 in case of error
 *
 * description:	This routine connects to a daemon via a unix domain socket.
 */

int unixconnect(char *n)
{
	int i;
	int s;
	struct sockaddr_un a;
	struct stat stb;

	/* create socket */

	if(UNLIKELY((s=socket(PF_UNIX,SOCK_STREAM,0))==-1))
		BAD("socket: %s",strerror(errno));
	fcntl(s,F_SETFD,FD_CLOEXEC);

	/* prepare connect */

	memset(&a,0,sizeof(a));
	a.sun_family=AF_UNIX;
	strncpy(a.sun_path,n,sizeof(a.sun_path)-1);
	a.sun_path[sizeof(a.sun_path)-1]=0;

	/* assert that the given path exists and is a socket */

	if(UNLIKELY(lstat(a.sun_path,&stb)))
	{
		close(s);
		BAD("%s does not exist",a.sun_path);
	}

	if(UNLIKELY(!S_ISSOCK(stb.st_mode)))
	{
		close(s);
		BAD("%s is not a socket",a.sun_path);
	}

	/* assert that the socket can be accessed */

	if(UNLIKELY(access(a.sun_path,R_OK|W_OK)))
	{
		close(s);
		BAD("%s can't be accessed",a.sun_path);
	}

	/* connect to daemon */

	if(UNLIKELY(connect(s,(struct sockaddr *)(&a),sizeof(a))))
	{
		i=errno;
		close(s);
		BAD("connect: %s",strerror(i));
	}

	/* return socket descriptor */

	return s;
}
